#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.


export SYNO_LDAP_ROOT="/var/packages/DirectoryServer/target/"
export SYNO_SSO_ROOT="$SYNO_LDAP_ROOT/simplesamlphp/www/"
export SYNO_SSO_ENTRY_ID="syno.entryid"
export SYNO_SESSION_DIR="$SYNO_LDAP_ROOT/var"
export SYNO_CGI_HEADER="@SYNO_CGI@"  # Lines start with this should be HTTP header


# Initialize global variables passed to PHP.
# Get corresponding PHP script filename in $PHPScripts.
# Create session by PHP if PHPSESSID not given in cookies.
SYNOCGIInit()
{
	local cookie="$HTTP_COOKIE" session="no" session_id="" item=""
	local php_option="-d open_basedir=$SYNO_SESSION_DIR -d session.save_path=$SYNO_SESSION_DIR"
	[ -z "$QUERY_STRING" ] && QUERY_STRING="language=en" || QUERY_STRING="$QUERY_STRING&language=en"

	read -n "$CONTENT_LENGTH" SYNO_POST_STRING  # FIXME use cat instead? from bashlib
	0<&- # close stdin (because scgi doesn't close stdin itself)
	export SYNO_POST_STRING="$SYNO_POST_STRING"

	PHPScript="$(basename "$0" | sed 's/\.cgi$//g').php"

	for item in $HTTP_COOKIE; do
		[ "${item:0:10}" = "PHPSESSID=" ] && session="yes"
	done
	mkdir -p $SYNO_SESSION_DIR
	chmod 700 $SYNO_SESSION_DIR
	if [ "x$session" = "xno" ]; then
		session_id="$(/usr/bin/php $php_option -r 'session_start();print session_id();')"
		cookie="PHPSESSID=$session_id; $cookie"
		echo "Set-Cookie: PHPSESSID=$session_id; path=/"
	fi
	export HTTP_COOKIE="$cookie"
}

# Execute corresponding PHP script and put result in $PHP_OUTPUT_BUFFER.
SYNOCGIWrap()
{
	local open_basedir="$SYNO_LDAP_ROOT/simplesamlphp:$SYNO_SESSION_DIR:/usr/local/etc/certificate/DirectoryServer/slapd"
	local php_option="-d open_basedir=$open_basedir -d session.save_path=$SYNO_SESSION_DIR -d extension=ldap.so"
	PHP_OUTPUT_BUFFER="$(/usr/bin/php $php_option -f "$PHPScript")"
}

# Print result of execution of corresponding PHP scripts (include HTTP headers).
# "Content-type: text/html" will be print if $PHP_OUTPUT_BUFFER contains no HTTP header.
SYNOCGIExit()
{
	local header_callback="$SYNO_HTTP_HEADER_CALLBACK"
	local content_callback="$SYNO_HTTP_CONTENT_CALLBACK"
	local start="no" end="no" line=
	echo "$PHP_OUTPUT_BUFFER" | while read line; do
		if echo "$line" | grep -q "^$SYNO_CGI_HEADER"; then
			start="yes"
			line=${line:${#SYNO_CGI_HEADER}}
			[ -z "$header_callback" ] && echo "$line" || eval $header_callback \'$line\'
		else
			if [ "x$end" = "xno" ]; then
				[ "x$start" = "xno" ] && echo "Content-type: text/html"
				echo ""  # Separate header and content
			fi
			end="yes"
			[ -z "$content_callback" ] && echo "$line" || eval $content_callback \'$line\'
		fi
	done
}

# Callback function to discard HTTP content received from PHP.
HttpContentDiscard()
{
	return
}

# vim:ft=sh ts=4 sts=4 sw=4
